@@ -327,7 +327,7 @@ static void loadSSL(void) {
   };
   for (unsigned i = 0; i < sizeof(symbols)/sizeof(symbols[0]); i++) {
     if (!(*symbols[i].var = loadSymbol(path_libssl, symbols[i].fn))) {
-      debug("SSL: failed to load SSL support. Could not find \"%s\"",
+      debug("[ssl] Failed to load SSL support. Could not find \"%s\"!",
             symbols[i].fn);
       for (unsigned j = 0; j < sizeof(symbols)/sizeof(symbols[0]); j++) {
         *symbols[j].var = NULL;
@@ -342,7 +342,7 @@ static void loadSSL(void) {
 
   SSL_library_init();
   dcheck(!ERR_peek_error());
-  debug("SSL: loaded SSL suppport");
+  debug("[ssl] Loaded SSL suppport...");
 }
 #endif
 
@@ -379,12 +379,12 @@ int serverSupportsSSL(void) {
 #if defined(HAVE_OPENSSL)
 static void sslGenerateCertificate(const char *certificate,
                                    const char *serverName) {
-  debug("SSL: auto-generating missing certificate \"%s\" for \"%s\"",
+  info("[ssl] Auto-generating missing certificate \"%s\" for \"%s\"...",
         certificate, serverName);
 
   pid_t pid       = fork();
   if (pid == -1) {
-    warn("SSL: failed to generate self-signed certificate \"%s\"", certificate);
+    warn("[ssl] Failed to generate self-signed certificate \"%s\"!", certificate);
   } else if (pid == 0) {
     int fd        = NOINTR(open("/dev/null", O_RDONLY));
     check(fd != -1);
@@ -401,14 +401,16 @@ static void sslGenerateCertificate(const char *certificate,
     if (execlp("openssl", "openssl", "req", "-x509", "-nodes", "-days", "7300",
                "-newkey", "rsa:2048", "-keyout", certificate, "-out", certificate,
                "-subj", subject, (char *)NULL) < 0) {
-      warn("SSL: failed to generate self-signed certificate \"%s\"", certificate);
+      warn("[ssl] Failed to generate self-signed certificate \"%s\"!", certificate);
       free(subject);
     }
   } else {
     int status;
     check(NOINTR(waitpid(pid, &status, 0)) == pid);
     if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
-      warn("SSL: failed to generate self-signed certificate \"%s\"", certificate);
+      warn("[ssl] Failed to generate self-signed certificate \"%s\"!", certificate);
+    } else {
+      info("[ssl] Certificate succesfully generated.");
     }
   }
 }
@@ -657,7 +659,7 @@ static SSL_CTX *sslMakeContext(void) {
   SSL_CTX_set_tmp_ecdh(context, ecKey);
   EC_KEY_free(ecKey);
 
-  debug("SSL: support for PFS enabled...");
+  debug("[ssl] Support for PFS enabled...");
 #endif
 
   check(SSL_CTX_set_cipher_list(context,
@@ -672,7 +674,7 @@ static SSL_CTX *sslMakeContext(void) {
 
   SSL_CTX_set_info_callback(context, sslInfoCallback);
 
-  debug("SSL: server context succesfully initialized...");
+  debug("[ssl] Server context succesfully initialized...");
   return context;
 }
 #endif
@@ -689,7 +691,7 @@ static int sslSNICallback(SSL *sslHndl, int *al ATTR_UNUSED,
   }
   struct HttpConnection *http =
                             (struct HttpConnection *)SSL_get_app_data(sslHndl);
-  debug("SSL: received SNI callback for virtual host \"%s\" from \"%s:%d\"",
+  debug("[ssl] Received SNI callback for virtual host \"%s\" from \"%s:%d\"...",
         name, httpGetPeerName(http), httpGetPort(http));
   char *serverName;
   check(serverName        = malloc(strlen(name)+2));
@@ -726,7 +728,7 @@ static int sslSNICallback(SSL *sslHndl, int *al ATTR_UNUSED,
         // the default certificate, instead.
         sslSetCertificateFromFile(context, certificate);
       } else {
-        warn("SSL: could not find matching certificate \"%s\" for \"%s\"",
+        warn("[ssl] Could not find matching certificate \"%s\" for \"%s\"",
              certificate, serverName + 1);
         SSL_CTX_free(context);
         context           = ssl->sslContext;
@@ -803,7 +805,7 @@ void sslSetCertificate(struct SSLSupport *ssl, const char *filename,
         sslGenerateCertificate(defaultCertificate, he->h_name);
       } else {
         if (h_err) {
-          warn("SSL: error getting host information: \"%s\".", hstrerror(h_err));
+          warn("[ssl] Error getting host information: \"%s\".", hstrerror(h_err));
         }
         sslGenerateCertificate(defaultCertificate, hostname);
       }
@@ -812,7 +814,7 @@ void sslSetCertificate(struct SSLSupport *ssl, const char *filename,
     }
   }
   if (sslSetCertificateFromFile(ssl->sslContext, defaultCertificate) < 0) {
-    fatal("SSL: cannot read valid certificate from \"%s\". "
+    fatal("[ssl] Cannot read valid certificate from \"%s\"! "
           "Check file permissions and file format.", defaultCertificate);
   }
  valid_certificate:
@@ -878,7 +880,7 @@ void sslSetCertificateFd(struct SSLSupport *ssl, int fd) {
   ssl->sslContext = sslMakeContext();
   char *filename = sslFdToFilename(fd);
   if (!sslSetCertificateFromFd(ssl->sslContext, fd)) {
-    fatal("SSL: cannot read valid certificate from %s. Check file format.",
+    fatal("[ssl] Cannot read valid certificate from %s. Check file format.",
           filename);
   }
   free(filename);
@@ -1051,7 +1053,7 @@ void sslFreeHndl(SSL **sslHndl) {
         // We do not know, how to fix this situation. Something must have
         // changed in the OpenSSL internals. Either, this is a new bug, or
         // somebody fixed the code in a way that we did not anticipate.
-        fatal("SSL: unexpected corruption of OpenSSL data structures");
+        fatal("[ssl] Unexpected corruption of OpenSSL data structures");
       }
     }
     SSL_free(*sslHndl);
